#include <stdbool.h>
#include <string.h>
#include <util/delay.h>     /* for _delay_ms() */
#include "usbdrv.h"

#include <stdio.h>
#include <motelib/uart.h> 
#include <motelib/radio.h> 
#include <motelib/system.h>
#include <motelib/timer.h>
#include <motelib/led.h>

#define MOD_CONTROL_LEFT    (1<<0)
#define MOD_SHIFT_LEFT      (1<<1)
#define MOD_ALT_LEFT        (1<<2)
#define MOD_GUI_LEFT        (1<<3)
#define MOD_CONTROL_RIGHT   (1<<4)
#define MOD_SHIFT_RIGHT     (1<<5)
#define MOD_ALT_RIGHT       (1<<6)
#define MOD_GUI_RIGHT       (1<<7)

#define KEY_A       4
#define KEY_B       5
#define KEY_C       6
#define KEY_D       7
#define KEY_E       8
#define KEY_F       9
#define KEY_G       10
#define KEY_H       11
#define KEY_I       12
#define KEY_J       13
#define KEY_K       14
#define KEY_L       15
#define KEY_M       16
#define KEY_N       17
#define KEY_O       18
#define KEY_P       19
#define KEY_Q       20
#define KEY_R       21
#define KEY_S       22
#define KEY_T       23
#define KEY_U       24
#define KEY_V       25
#define KEY_W       26
#define KEY_X       27
#define KEY_Y       28
#define KEY_Z       29
#define KEY_1       30
#define KEY_2       31
#define KEY_3       32
#define KEY_4       33
#define KEY_5       34
#define KEY_6       35
#define KEY_7       36
#define KEY_8       37
#define KEY_9       38
#define KEY_0       39

#define KEY_F1      58
#define KEY_F2      59
#define KEY_F3      60
#define KEY_F4      61
#define KEY_F5      62
#define KEY_F6      63
#define KEY_F7      64
#define KEY_F8      65
#define KEY_F9      66
#define KEY_F10     67
#define KEY_F11     68
#define KEY_F12     69

// Mouse with 3 buttons, middle button is wheel
PROGMEM const char usbHidReportDescriptor[] PROGMEM = 
{
	// start of mouse report descriptor
	0x05, 0x01,	   // USAGE_PAGE (Generic Desktop)
	0x09, 0x02,	   // USAGE (Mouse)
	0xa1, 0x01,	   // COLLECTION (Application)
	0x09, 0x01,	   //   USAGE (Pointer)
	0xa1, 0x00,	   //   COLLECTION (Physical)
	0x85, 0x01,	   //   REPORT_ID (1)
	0x05, 0x09,	   //	 USAGE_PAGE (Button)
	0x19, 0x01,	   //	 USAGE_MINIMUM (Button 1)
	0x29, 0x03,	   //	 USAGE_MAXIMUM (Button 3)
	0x15, 0x00,	   //	 LOGICAL_MINIMUM (0)
	0x25, 0x01,	   //	 LOGICAL_MAXIMUM (1)
	0x95, 0x03,	   //	 REPORT_COUNT (3)
	0x75, 0x01,	   //	 REPORT_SIZE (1)
	0x81, 0x02,	   //	 INPUT (Data,Var,Abs)
	0x95, 0x01,	   //	 REPORT_COUNT (1)
	0x75, 0x05,	   //	 REPORT_SIZE (5)
	0x81, 0x03,	   //	 INPUT (Cnst,Var,Abs)
	0x05, 0x01,	   //	 USAGE_PAGE (Generic Desktop)
	0x09, 0x30,	   //	 USAGE (X)
	0x09, 0x31,	   //	 USAGE (Y)
	0x09, 0x38,	   //	 USAGE (Wheel)
	0x15, 0x81,	   //	 LOGICAL_MINIMUM (-127)
	0x25, 0x7f,	   //	 LOGICAL_MAXIMUM (127)
	0x75, 0x08,	   //	 REPORT_SIZE (8)
	0x95, 0x03,	   //	 REPORT_COUNT (3)
	0x81, 0x06,	   //	 INPUT (Data,Var,Rel)
	0x05, 0x0c,	   //	 USAGE_PAGE (Consumer Devices)
	0x0a, 0x38, 0x02, //	 USAGE (Undefined)
	0x95, 0x01,	   //	 REPORT_COUNT (1)
	0x81, 0x06,	   //	 INPUT (Data,Var,Rel)
	0xc0,          //   END_COLLECTION
	0xc0,          // END_COLLECTION

	0x05, 0x01,   // USAGE_PAGE (Generic Desktop)
	0x09, 0x06,   // USAGE (Keyboard)
	0xa1, 0x01,   // COLLECTION (Application)
	0x85, 0x02,   //   REPORT_ID (2)
	0x05, 0x07,   //   USAGE_PAGE (Keyboard)
	0x19, 0xe0,   //   USAGE_MINIMUM (Keyboard LeftControl)
	0x29, 0xe7,   //   USAGE_MAXIMUM (Keyboard Right GUI)
	0x15, 0x00,   //   LOGICAL_MINIMUM (0)
	0x25, 0x01,   //   LOGICAL_MAXIMUM (1)
	0x75, 0x01,   //   REPORT_SIZE (1)
	0x95, 0x08,   //   REPORT_COUNT (8)
	0x81, 0x02,   //   INPUT (Data,Var,Abs)
	0x95, 0x01,   //   REPORT_COUNT (1)
	0x75, 0x08,   //   REPORT_SIZE (8)
	0x81, 0x03,   //   INPUT (Cnst,Var,Abs)
	0x95, 0x05,   //   REPORT_COUNT (5)
	0x75, 0x01,   //   REPORT_SIZE (1)
	0x05, 0x08,   //   USAGE_PAGE (LEDs)
	0x19, 0x01,   //   USAGE_MINIMUM (Num Lock)
	0x29, 0x05,   //   USAGE_MAXIMUM (Kana)
	0x91, 0x02,   //   OUTPUT (Data,Var,Abs)
	0x95, 0x01,   //   REPORT_COUNT (1)
	0x75, 0x03,   //   REPORT_SIZE (3)
	0x91, 0x03,   //   OUTPUT (Cnst,Var,Abs)
	0x95, 0x06,   //   REPORT_COUNT (6)
	0x75, 0x08,   //   REPORT_SIZE (8)
	0x15, 0x00,   //   LOGICAL_MINIMUM (0)
	0x25, 0x65,   //   LOGICAL_MAXIMUM (101)
	0x05, 0x07,   //   USAGE_PAGE (Keyboard)
	0x19, 0x00,   //   USAGE_MINIMUM (Reserved (no event indicated))
	0x29, 0x65,   //   USAGE_MAXIMUM (Keyboard Application)
	0x81, 0x00,   //   INPUT (Data,Ary,Abs)
	0xc0,         // END_COLLECTION

};



typedef struct
{
	uint8_t id;
    uint8_t  buttons;
    int8_t   dx,dy;
    int8_t   dwheel;
} report_t;

static report_t reportBuffer;
static uchar reportBuffer2[5] = {2,0,0,0,0};    /* keyboard buffer for HID reports */
static uint8_t  idleRate;   /* repeat rate for keyboards, never used for mice */
static bool newRequest,newRequest2;
static Timer t;

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;

    /* class request type */
    if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS)
    {
        if(rq->bRequest == USBRQ_HID_GET_REPORT)
        {
			if (rq->wValue.bytes[0] == 1) {
				usbMsgPtr = (void *)&reportBuffer;
				return sizeof(reportBuffer);
			} else if (rq->wValue.bytes[0] == 2) {
				usbMsgPtr = reportBuffer2;
				return sizeof(reportBuffer2);
			}
			return 0;
//            usbMsgPtr = (void *)&reportBuffer;
//            return sizeof(reportBuffer);
        } 
		else if (rq->bRequest == USBRQ_HID_SET_REPORT) 
		{
			/* wValue: ReportType (highbyte), ReportID (lowbyte) */
			/* we have no output/feature reports */
			return 0;
		}
        else if (rq->bRequest == USBRQ_HID_GET_IDLE)
        {
            usbMsgPtr = &idleRate;
            return 1;
        }
        else if (rq->bRequest == USBRQ_HID_SET_IDLE)
        {
            idleRate = rq->wValue.bytes[1];
        }
    }
    else
    {
        /* no vendor specific requests implemented */
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

void loop()
{
    usbPoll();

    if (usbInterruptIsReady())
    {
		if(newRequest)
		{
		    ledToggle(0);
		    usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
			reportBuffer.dx = 0;
			reportBuffer.dy = 0;
			reportBuffer.dwheel = 0;
		    // clear report buffer and flag
		    //memset(&reportBuffer, 0, sizeof(reportBuffer));
		    newRequest = false;
		}
		else if(newRequest2)
		{
			//ledToggle(0);
		    usbSetInterrupt((void *)&reportBuffer2, sizeof(reportBuffer2));
			_delay_ms(300);
		    // clear report buffer and flag
			//reportBuffer2[3] = 0;
		    //usbSetInterrupt((void *)&reportBuffer2, sizeof(reportBuffer2));
		    newRequest2 = false;
		}
    }
}

void receive(Address src, MessageType type, void *msg, uint8_t len) 
{ 
	if(src == 0)
	{
		int16_t event = *((int16_t*)msg);
		if(type >= 1 && type <= 6)
			newRequest= true;		
		if (type == 1 && event == 1)
			reportBuffer.buttons |= (int8_t)event;		
		if (type == 1 && event == 0) 
			reportBuffer.buttons &= (~(int8_t)event)<<1;		
		if (type == 2 && event == 1) 
			reportBuffer.buttons |= (int8_t)event<<1;
		if (type == 2 && event == 0) 
			reportBuffer.buttons &= 0xfd;
		if (type == 3 && event == 1) 
			reportBuffer.buttons |= event<<2;
		if (type == 3 && event == 0) 
			reportBuffer.buttons &= 0xfb;
		if (type == 4) 
			reportBuffer.dx = event;
		if (type == 5) 
			reportBuffer.dy = event;
		if (type == 6) 
			reportBuffer.dwheel = event;
		if (type == 99)
		{
			uint8_t result = ((char*)msg)[0];
		    if(result==1){
		    	reportBuffer2[3] = KEY_Q;
		    	printf("Q\n");
		    }
		    else if(result==2){
		    	reportBuffer2[3] = KEY_W;
		    	printf("W\n");
		    }
		    else if(result==3){
		    	reportBuffer2[3] = KEY_E;
		    	printf("E\n");
		    }
		    else if(result==4){
		    	reportBuffer2[3] = KEY_R;
		    	printf("R\n");
		    }
		    else if(result==0){
		    	reportBuffer2[3] = 0;
		    	printf("0\n");
		    }
		    newRequest2 = true;
		}
	}
}

void boot()
{
    memset(&reportBuffer, 0, sizeof(reportBuffer));
	reportBuffer.id = 1;
    newRequest = false;
    newRequest2 = false;
    usbInit();
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

	uartEnable(true,true);
	radioSetRxHandler(receive); 
    setLoopRoutine(loop);
}
